import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Col, Tooltip, notification, Typography, message } from 'antd'
import moment from 'moment'

// import Api from '@/api'
import asyncComponent from '@/utils/asyncComponent'
import { getMark } from '@/utils/utils.js'
import MkIcon from '@/components/mk-icon'
import MKEmitter from '@/utils/events.js'
import Encrypts from '@/components/encrypts'
import './index.scss'

moment.suppressDeprecationWarnings = true
const { Paragraph } = Typography
const NormalButton = asyncComponent(() => import('@/tabviews/zshare/actionList/normalbutton'))
const ExcelInButton = asyncComponent(() => import('@/tabviews/zshare/actionList/excelInbutton'))
const ExcelOutButton = asyncComponent(() => import('@/tabviews/zshare/actionList/exceloutbutton'))
const PopupButton = asyncComponent(() => import('@/tabviews/zshare/actionList/popupbutton'))
const TabButton = asyncComponent(() => import('@/tabviews/zshare/actionList/tabbutton'))
const NewPageButton = asyncComponent(() => import('@/tabviews/zshare/actionList/newpagebutton'))
const ChangeUserButton = asyncComponent(() => import('@/tabviews/zshare/actionList/changeuserbutton'))
const PrintButton = asyncComponent(() => import('@/tabviews/zshare/actionList/printbutton'))
const FuncMegvii = asyncComponent(() => import('@/tabviews/zshare/actionList/funcMegvii'))
const FuncZip = asyncComponent(() => import('@/tabviews/zshare/actionList/funczip'))
const ExportPdf = asyncComponent(() => import('@/tabviews/zshare/actionList/exportPdf'))
const FuncButton = asyncComponent(() => import('@/tabviews/zshare/actionList/funcbutton'))
const EditLine = asyncComponent(() => import('@/tabviews/zshare/actionList/editLine'))
const BarCode = asyncComponent(() => import('@/components/barcode'))
const QrCode = asyncComponent(() => import('@/components/qrcode'))
const MkProgress = asyncComponent(() => import('@/components/mkProgress'))
const Video = asyncComponent(() => import('@/components/video'))
const MkPicture = asyncComponent(() => import('@/components/mkPicture'))
const PicRadio = {
  '4:3': '75%', '3:2': '66.67%', '16:9': '56.25%', '2:1': '50%', '3:1': '33.33%', '4:1': '25%',
  '5:1': '20%', '6:1': '16.67%', '7:1': '14.29%', '8:1': '12.5%', '9:1': '11.11%',
  '10:1': '10%', '3:4': '133.33%', '2:3': '150%', '9:16': '177.78%'
}

class TextCell extends Component {
  componentDidMount() {
    if (this.node && this.node.scrollHeight > this.node.offsetHeight) {
      this.node.style.display = 'block'
    }
  }

  componentDidUpdate() {
    if (this.node) {
      if (this.node.scrollHeight > this.node.offsetHeight) {
        this.node.style.display = 'block'
      } else {
        this.node.style.display = 'flex'
      }
    }
  }

  render() {
    const { card, className, value } = this.props

    let lineStyle = {height: card.innerHeight}
    lineStyle.display = 'flex'
    lineStyle.alignItems = card.alignItems
    lineStyle.justifyContent = card.style.textAlign || 'left'
  
    return (
      <div ref={ref => this.node = ref} className={className} style={lineStyle}>{value}</div>
    )
  }
}

class CardCellComponent extends Component {
  static propTpyes = {
    cards: PropTypes.object,         // 菜单配置信息
    cardCell: PropTypes.object,
    data: PropTypes.object,
    syncData: PropTypes.array,
    elements: PropTypes.array,       // 元素集
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props.data), fromJS(nextProps.data)) || (nextProps.syncData ? !is(fromJS(this.props.syncData), fromJS(nextProps.syncData)) : false)
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  openNewView = (e, card) => {
    const { cardCell, data, cards } = this.props

    e.stopPropagation()

    if (card.anchors) {
      let id = card.anchors[card.anchors.length - 1]
      let node = document.getElementById('anchor' + id)
      node && node.scrollIntoView({behavior: 'smooth', block: 'center', inline: 'nearest'})
    }

    if (card.linkType === 'qywx') return
    if (card.linkType === 'linkmenu') {
      if (card.linkThdMenu) {
        let __param = {
          $BID: data.$$uuid || ''
        }
  
        if (card.field) {
          __param.$searchkey = card.field
          __param.$searchval = data[card.field] || ''
        }
      
        Object.keys(data).forEach(key => {
          if (/^\$/.test(key)) return
          if (key === 'children') return
          __param[key] = data[key]
        })

        let tabmenu = card.linkThdMenu
    
        tabmenu.param = __param
    
        MKEmitter.emit('modifyTabs', tabmenu, true)
      }
      return
    }
    
    let url = ''

    if (card.link === 'static') {
      url = card.linkurl
    } else {
      url = data[card.linkurl]
    }

    if (!url) {
      notification.warning({
        top: 92,
        message: '链接地址不存在！',
        duration: 5
      })
      return
    }

    if (card.linkType === 'tel') {
      window.open('tel:' + url)
      return
    } else if (card.linkType === 'email') {
      let _url = 'mailto:' + url
      let fullName = sessionStorage.getItem('Full_Name') || ''
      if (fullName) {
        _url = _url + `?subject=来自${fullName}的邮件`
      }
      window.open(_url)
      return
    } else if (card.linkType === 'other' && /^@menuid@/ig.test(url)) {
      return
    }

    // positecgroup
    // if (/^sso$/ig.test(url)) {
    //   if (!data.LinkUrl1) {
    //     notification.warning({
    //       top: 92,
    //       message: '链接地址不存在！',
    //       duration: 5
    //     })
    //     return
    //   }

    //   let _url = data.LinkUrl1
    //   if (/index\.html/ig.test(_url)) {
    //     _url = _url.replace(/index\.html.*/ig, '')
    //   } else if (!/\/$/ig.test(_url)) {
    //     _url = _url + '/'
    //   }

    //   let key = (() => {
    //     let uuid = []
    //     let timestamp = new Date().getTime()
    //     let _options = '0123456789abcdefghigklmnopqrstuv'
    //     for (let i = 0; i < 19; i++) {
    //       uuid.push(_options.substr(Math.floor(Math.random() * 0x20), 1))
    //     }
    //     uuid = timestamp + uuid.join('')
    //     return uuid
    //   })()

    //   let _param = {
    //     func: 'webapi_scan_binding_key',
    //     binding_type: 'mk',
    //     scan_type: 'toggle',
    //     scan_appkey: data.scan_appkey || '',
    //     id: key
    //   }
  
    //   Api.getSystemConfig(_param).then(res => {
    //     if (!res.status) {
    //       notification.warning({
    //         top: 92,
    //         message: res.message,
    //         duration: 5
    //       })
    //     } else if (res.thd_party_appid && res.thd_party_member_id && res.thd_party_openid) {
    //       let href = _url + 'admin/index.html#/ssologin/' + window.btoa(window.encodeURIComponent(JSON.stringify({
    //         appid: res.thd_party_appid,
    //         memberId: res.thd_party_member_id,
    //         openid: res.thd_party_openid,
    //         key: key
    //       })))
  
    //       window.open(href)
    //     } else {
    //       notification.warning({
    //         top: 92,
    //         message: '信息缺失，请联系管理员！',
    //         duration: 5
    //       })
    //     }
    //   })
    //   return
    // }

    if (/^(http|\/\/).+(.txt|.doc|.docx|.pdf|.xlsx|.xls|.zip|.rar|.ppt)$/i.test(url) || card.linkType === 'download') {
      let name = ''
      if (card.datatype === 'static') {
        name = card.value || ''
      } else if (data.hasOwnProperty(card.field)) {
        name = data[card.field] + ''
      }
      name = name.replace(/\.{1}[^.]*$/g, '')
      name = name ? name + url.replace(/.+\./g, '.') : ''

      let d = document.createElement('a')
      d.href = url
      d.setAttribute('download', name)
      d.setAttribute('target', '_blank')
      document.body.appendChild(d)
      d.click()
      d.remove()
    } else {
      let Id = data.$$uuid || ''

      if (cards.subtype === 'propcard' && cardCell) {
        Id = cardCell.setting.primaryId || ''
      }

      if (card.joint === 'true') {
        let con = '?'
  
        if (/\?/ig.test(url)) {
          con = '&'
        }
  
        url = url + `${con}id=${Id}&appkey=${window.GLOB.appkey}&userid=${sessionStorage.getItem('UserID')}&LoginUID=${sessionStorage.getItem('LoginUID') || ''}`
      } else if (/@/.test(url)) {
        url = url.replace(/@id@/ig, Id)
        url = url.replace(/@appkey@/ig, window.GLOB.appkey)
        url = url.replace(/@userid@/ig, sessionStorage.getItem('UserID'))
        url = url.replace(/@LoginUID@/ig, sessionStorage.getItem('LoginUID'))

        Object.keys(data).forEach(key => {
          if (/^\$/.test(key)) return
          if (key === 'children') return
          let reg = new RegExp('@' + key + '@', 'ig')
          url = url.replace(reg, data[key])
        })
      }
  
      window.open(url)
    }
  }

  getColor = (marks) => {
    const { data } = this.props
    let color = ''

    marks.some(mark => {
      let originVal = data[mark.field[0]] + ''
      let contrastVal = ''
      let result = false

      if (mark.field[1] === 'static') {
        contrastVal = mark.contrastValue + ''
      } else {
        contrastVal = data[mark.field[2]] + ''
      }

      if (mark.match === '=') {
        result = originVal === contrastVal
      } else if (mark.match === '!=') {
        result = originVal !== contrastVal
      } else if (mark.match === 'like') {
        result = originVal.indexOf(contrastVal) > -1
      } else if (mark.match === '>') {
        result = parseFloat(originVal) > parseFloat(contrastVal)
      } else if (mark.match === '<') {
        result = parseFloat(originVal) < parseFloat(contrastVal)
      }

      if (result) {
        color = mark.color
      }
      return result
    })

    return color
  }

  getContent = () => {
    const { data, cards, elements } = this.props

    let contents = []

    elements.forEach(card => {
      let _style_ = null

      if (card.style && card.style.clear === 'left') {
        _style_ = {clear: 'left'}
      } else if (card.style && card.style.clear === 'right') {
        _style_ = {float: 'right'}
      }

      if (card.eleType === 'sequence') {
        let _style = {}
        let className = ''
        if (card.marks) {
          _style.width = card.innerHeight
          _style.height = card.innerHeight
          _style.lineHeight = card.innerHeight + 'px'
  
          let mark = getMark(card.marks, data, _style)

          className = mark.signType
        }
        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
            <div style={card.style}>
              <div className={'ant-mk-text line1' + className} style={{height: card.innerHeight || 'auto'}}><span className="sequence-wrap" style={_style}>{card.prefix || ''}{data.$Index || ''}{card.postfix || ''}</span></div>
            </div>
          </div>
        )
      } else if (card.eleType === 'text') {
        let val = ''
        let _style = {...card.style}
  
        if (card.datatype === 'static') {
          val = card.value || ''
          if (/@.+@/g.test(val)) {
            if (/@username@|@fullName@|@mk_city@|@appname@|@bid@/ig.test(val)) {
              let userName = sessionStorage.getItem('User_Name') || ''
              let fullName = sessionStorage.getItem('Full_Name') || ''
              let city = sessionStorage.getItem('city') || ''
              let appname = sessionStorage.getItem('appname') || ''
              let bid = data.$$BID || ''
              val = val.replace(/@username@/ig, userName).replace(/@fullName@/ig, fullName).replace(/@mk_city@/ig, city).replace(/@appname@/ig, appname).replace(/@bid@/ig, bid)
            } else if (/@month@/ig.test(val)) {
              val = val.replace(/@month@/ig, new Date().toLocaleString('en-US', { month: 'long' }))
            } else if (/@week@/ig.test(val)) {
              val = val.replace(/@week@/ig, (() => {
                let day = new Date().getDay()
                let weeks = ['星期日', '星期一', '星期二', '星期三', '星期四', '星期五', '星期六']
                return weeks[day]
              })())
            } else if (/@day@/ig.test(val)) {
              val = val.replace(/@day@/ig, (() => {
                let day = new Date().getDate()
                return day < 10 ? '0' + day : day
              })())
            }
          }

          if (card.noVField) {
            if (!data[card.noVField]) {
              val = ''
            } else if (card.noVExtra && new RegExp('^(' + card.noVExtra.split(',').join('|') + ')$', 'ig').test(data[card.noVField])) {
              val = ''
            }
          }
        } else if (data.hasOwnProperty(card.field)) {
          val = data[card.field] + ''
          if (card.noVExtra && val && new RegExp('^(' + card.noVExtra.split(',').join('|') + ')$', 'ig').test(val)) {
            val = ''
          }
        }
  
        if (val && card.format && card.format !== 'encryption') {
          let _val = null
  
          if (val < '1949-10-02') {
            val = ''
          } else if (card.format === 'calendar1') {
            _val = moment(val).calendar(null, {
              sameDay: '[今天] ahh:mm',
              nextDay: '[明天] ahh:mm',
              nextWeek: 'MM月DD日 ahh:mm',
              lastDay: '[昨天] ahh:mm',
              lastWeek: 'dddd ahh:mm',
              sameElse: 'MM月DD日 ahh:mm'
            })
          } else if (card.format === 'calendar2') {
            let time = new Date(val).getTime()
            if (!isNaN(time)) {
              time = parseInt(time / 60000)                                     // 时间值
              let now = parseInt(new Date().getTime() / 60000)                  // 当前时间值
              let start = new Date(new Date().toDateString()).getTime() / 60000 // 今天零点时间值
              let split = now - time
  
              if (split < 0) { // 时间值在当前时间之后
                _val = moment(val).format('MM月DD日 HH:mm')
              } else if (split < 3) {
                _val = '刚刚'
              } else if (split < 5) {
                _val = '3分钟前'
              } else if (split < 10) {
                _val = '5分钟前'
              } else if (split < 20) {
                _val = '10分钟前'
              } else if (split < 30) {
                _val = '20分钟前'
              } else if (split < 60) {
                _val = '30分钟前'
              } else if (split < 420 || time > start) { // 7小时内或时间值在今天零点后
                _val = parseInt(split / 60) + '小时前'
              } else {                                  // 时间值在今天零点之前
                let _day = parseInt((start - time) / (24 * 60)) + 1
                if (_day === 1) {
                  _val = '昨天'
                } else if (_day <= 30) {
                  _val = _day + '天前'
                } else {
                  _val = moment(val).format('MM月DD日 HH:mm')
                }
              }
            }
          } else {
            _val = moment(val).format(card.format)
          }

          if (_val && _val !== 'Invalid date') {
            val = _val
          }
        }

        if (!val && card.noValue === 'hide') { // 空值隐藏
          return null
        }
  
        if (val !== '') {
          let orival = val

          if (card.linkType === 'download') {
            let url = ''
  
            if (card.link === 'static') {
              url = card.linkurl
            } else {
              url = data[card.linkurl]
            }
  
            if (/^(http|\/\/).+(.txt|.doc|.docx|.pdf|.xlsx|.xls|.zip|.rar)$/i.test(url)) {
              if (/pdf$/i.test(url)) {
                val = <><img src="./media/pdf.png" className="file-image" alt=""/> {val}</>
              } else if (/(.doc|.docx)$/i.test(url)) {
                val = <><img src="./media/word.png" className="file-image" alt=""/> {val}</>
              } else if (/(.xlsx|.xls)$/i.test(url)) {
                val = <><img src="./media/excel.png" className="file-image" alt=""/> {val}</>
              } else if (/(.zip|.rar)$/i.test(url)) {
                val = <><img src="./media/rar.png" className="file-image" alt=""/> {val}</>
              } else {
                val = <><img src="./media/txt.png" className="file-image" alt=""/> {val}</>
              }
            }
          }

          if (card.format === 'encryption') {
            val = <Encrypts value={val} />
          }
          if (card.fixStyle === 'alone') {
            let _s = {fontSize: card.fixSize, color: card.fixColor, marginLeft: card.fixLeft, marginRight: card.fixRight}
            val = <><span style={_s}>{card.prefix || ''}</span>{val}<span style={_s}>{card.postfix || ''}</span></>
          } else {
            val = <span>{card.prefix || ''}{val}{card.postfix || ''}</span>
          }

          if (card.copyable === 'true') {
            if (card.link || card.anchors) {
              let url = orival

              if (card.link === 'static') {
                url = card.linkurl
              } else if (card.link === 'dynamic') {
                url = data[card.linkurl]
              }

              val = <span>{val}<Paragraph style={{display: 'inline-block'}} onClick={(e) => e.stopPropagation()} copyable={{ text: url }}></Paragraph></span>
            } else {
              val = <Paragraph copyable={{ text: orival }}>{val}</Paragraph>
            }
          }
        } else if (card.fixStyle === 'alone') {
          let _s = {fontSize: card.fixSize, color: card.fixColor, marginLeft: card.fixLeft, marginRight: card.fixRight}
          val = <span style={_s}>{card.prefix || ''}{card.postfix || ''}</span>
        }
  
        let className = ''
        if (card.marks) {
          let mark = getMark(card.marks, data, _style)
  
          if (mark.icon) {
            if (mark.position === 'front') {
              val = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {val}</span>
            } else {
              val = <span>{val} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
            }
          } else if (mark.space) {
            val = <><span style={{float: 'left'}} dangerouslySetInnerHTML={{__html: mark.space}}></span>{val}</>
          } else if (mark.point) {
            if (mark.position === 'front') {
              val = <>{mark.point}{val}</>
            } else {
              val = <>{val}{mark.point}</>
            }
          }
          className = mark.signType
        }
    
        if (card.bgImage && data[card.bgImage]) {
          _style.backgroundImage = `url('${data[card.bgImage]}')`
        }

        if (!data.$disabled && (card.link || card.anchors)) {
          _style.cursor = 'pointer'
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div style={_style} onClick={(e) => {this.openNewView(e, card)}}>
                {card.alignItems ? <TextCell card={card} className={'ant-mk-text line' + (card.height || '') + className} value={val}/> : 
                <div className={'ant-mk-text line' + (card.height || '') + className} style={{height: card.innerHeight}}>{val}</div>}
              </div>
            </div>
          )
        } else {
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div style={_style}>
                {card.alignItems ? <TextCell card={card} className={'ant-mk-text line' + (card.height || '') + className} value={val}/> : 
                <div className={'ant-mk-text line' + (card.height || '') + className} style={{height: card.innerHeight}}>{val}</div>}
              </div>
            </div>
          )
        }
      } else if (card.eleType === 'number') {
        let val = ''
        let _style = {...card.style}
  
        if (card.datatype === 'static') {
          val = card.value
        } else if (data.hasOwnProperty(card.field)) {
          val = data[card.field]
        }
  
        if (!val && card.noValue === 'hide') { // 空值隐藏
          return null
        }
  
        if (!isNaN(val) && val !== '') {
          val = +val
          if (card.round) {
            val = Math.round(val * card.round) / card.round
          }
          if (card.format === 'percent') {
            val = val * 100
            if (!card.round) {
              val = +val.toFixed(2)
            }
          } else if (card.format === 'abs') {
            val = Math.abs(val)
          }
  
          if (card.round) {
            val = val.toFixed(card.decimal)
          } else {
            val = '' + val
          }

          if (card.format === 'percent' && (!card.postfix || card.postfix.indexOf('%') === -1)) {
            val = val + '%'
          } else if (card.format === 'thdSeparator') {
            val = val.replace(/\d{1,3}(?=(\d{3})+(\.\d*)?$)/g, '$&,')
          }
        }
  
        if (val !== '') {
          if (card.fixStyle === 'alone') {
            let _s = {fontSize: card.fixSize, color: card.fixColor, marginLeft: card.fixLeft, marginRight: card.fixRight}
            val = <><span style={_s}>{card.prefix || ''}</span>{val}<span style={_s}>{card.postfix || ''}</span></>
          } else {
            val = `${card.prefix || ''}${val}${card.postfix || ''}`
          }
        } else if (card.fixStyle === 'alone') {
          let _s = {fontSize: card.fixSize, color: card.fixColor, marginLeft: card.fixLeft, marginRight: card.fixRight}
          val = <span style={_s}>{card.prefix || ''}{card.postfix || ''}</span>
        }
        
        let className = ''
        if (card.marks) {
          let mark = getMark(card.marks, data, _style)
  
          if (mark.icon) {
            if (mark.position === 'front') {
              val = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {val}</span>
            } else {
              val = <span>{val} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
            }
          } else if (mark.space) {
            val = <><span style={{float: 'left'}} dangerouslySetInnerHTML={{__html: mark.space}}></span>{val}</>
          } else if (mark.point) {
            if (mark.position === 'front') {
              val = <>{mark.point}{val}</>
            } else {
              val = <>{val}{mark.point}</>
            }
          }
          className = mark.signType
        }

        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
            <div style={_style}>
              {card.alignItems ? <TextCell card={card} className={'ant-mk-text line' + (card.height || '') + className} value={val}/> : 
              <div className={'ant-mk-text line' + (card.height || '') + className} style={{height: card.innerHeight}}>{val}</div>}
            </div>
          </div>
        )
      } else if (card.eleType === 'icon') {
        let val = ''
        let icon = ''

        if (card.datatype === 'dynamic') {
          icon = data[card.field] || ''
        } else if (card.tipType === 'text') {
          icon = card.value
        } else {
          icon = card.icon
        }

        if (!icon && card.noValue === 'hide') { // 空值隐藏
          return null
        }
  
        if (data.hasOwnProperty(card.tooltip)) {
          val = data[card.tooltip]
        } else {
          val = card.tooltip
        }

        if (/\\n|\n/.test(val)) {
          val = val.replace(/(\\n|\n)$/, '')
        
          if (val) {
            val = val.split(/\\n|\n/)
        
            val = <div>{val.map((cell, i) => <div style={{marginBottom: 2}} key={i}>{cell}</div>)}</div>
          }
        }

        let _style = {...card.style}

        if (card.marks) {
          let _s = {}
          getMark(card.marks, data, _s)

          if (_s.color) {
            _style.color = _s.color
          }
        }

        if (card.tipType === 'text') {
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div style={_style}>
                {val ? <Tooltip title={val}>
                  <div className={'ant-mk-text line' + (card.height || '')} style={{height: card.innerHeight}}>{icon}</div>
                </Tooltip> : <div className={'ant-mk-text line' + (card.height || '')} style={{height: card.innerHeight}}>{icon}</div>}
              </div>
            </div>
          )
        } else {
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div style={_style}>
                {val ? <Tooltip title={val}>
                  <MkIcon className="ant-mk-icon" style={{height: card.innerHeight}} type={icon}/>
                </Tooltip> : <MkIcon className="ant-mk-icon" style={{height: card.innerHeight}} type={icon}/>}
              </div>
            </div>
          )
        }
      } else if (card.eleType === 'slider') {
        let val = 0
        let color = card.color
  
        if (card.datatype === 'static') {
          val = card.value
        } else if (data.hasOwnProperty(card.field)) {
          val = parseFloat(data[card.field])
          if (isNaN(val)) {
            val = 0
          }
        }
  
        val = val / card.maxValue * 100
        val = parseInt(val * 100) / 100
  
        if (card.marks) {
          let _color = this.getColor(card.marks)
          color = _color ? _color : color
        }
  
        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
            <div style={card.style}>
              <MkProgress value={val} config={card} color={color}/>
            </div>
          </div>
        )
      } else if (card.eleType === 'picture') {
        let _imagestyle = {}
        let _style = {margin: '0 auto', ...card.style}
        let url = ''
        if (card.maxWidth) {
          _style.maxWidth = card.maxWidth
          if (_style.marginLeft === '0px' && _style.marginRight === '0px') {
            delete _style.marginLeft
            delete _style.marginRight
          }
        }
  
        if (card.datatype === 'static') {
          url = card.url || ''
          if (url === '@icon@') {
            url = sessionStorage.getItem('avatar') || ''
          }
        } else {
          url = data[card.field] || ''
          url = url.replace(/@mywebsite@\//ig, window.GLOB.baseurl)
        }
  
        if (url === '' && card.noValue === 'hide') { // 空值隐藏
          return null
        }
  
        if (PicRadio[card.lenWidRadio]) {
          _imagestyle.paddingTop = PicRadio[card.lenWidRadio]
        } else {
          _imagestyle.paddingTop = '100%'
        }

        _imagestyle.borderRadius = _style.borderRadius || 0
        _imagestyle.backgroundSize = _style.backgroundSize || 'cover'
        _imagestyle.backgroundPosition = _style.backgroundPosition || 'center'
        _imagestyle.backgroundRepeat = _style.backgroundRepeat || 'no-repeat'
  
        if (_style.position === 'absolute') {
          _style.width = '100%'
        }
        
        let scale = url && card.scale === 'true'
        let urls = url ? url.split(',').filter(Boolean) : ['']
        
        if (card.link && !data.$disabled) {
          _style.cursor = 'pointer'
          urls.forEach((u, i) => {
            contents.push(<div className={'ant-col ant-col-' + card.width} key={card.uuid + i} style={_style_} span={card.width}>
              <div style={_style} onClick={(e) => {this.openNewView(e, card)}}>
                <MkPicture style={_imagestyle} lostTip={card.lostTip !== 'false'} scale={scale} url={u} urls={urls}/>
              </div>
            </div>)
          })
        } else {
          urls.forEach((u, i) => {
            contents.push(<div className={'ant-col ant-col-' + card.width} key={card.uuid + i} style={_style_} span={card.width}>
              <div style={_style}>
                <MkPicture style={_imagestyle} lostTip={card.lostTip !== 'false'} scale={scale} url={u} urls={urls}/>
              </div>
            </div>)
          })
        }
      } else if (card.eleType === 'splitline') {
        let _borderWidth = card.borderWidth === undefined ? 1 : card.borderWidth
        _style_ = _style_ || {}
        _style_.minHeight = _borderWidth
        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} span={card.width} style={_style_}>
            <div style={card.style}>
              <div className="ant-mk-splitline" style={{borderColor: card.color, borderWidth: _borderWidth}}></div>
            </div>
          </div>
        )
      } else if (card.eleType === 'barcode') {
        let val = ''
  
        if (card.datatype === 'static') {
          val = card.value
        } else if (data.hasOwnProperty(card.field)) {
          val = data[card.field] || ''
        }
  
        if (!val && card.noValue === 'hide') { // 空值隐藏
          return null
        }
  
        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
            <div style={card.style}>
              <div style={{height: card.innerHeight || 25}}>
                {val ? <BarCode card={card} value={val}/> : null}
              </div>
            </div>
          </div>
        )
      } else if (card.eleType === 'video') {
        let url = ''
  
        if (card.datatype === 'static') {
          url = card.url
        } else {
          url = data[card.field] || ''
        }
  
        if (!url && card.noValue === 'hide') { // 空值隐藏
          return null
        }
  
        let poster = ''
  
        if (card.posterType === 'dynamic') {
          poster = data[card.posterField] || ''
        } else {
          poster = card.posterUrl || ''
        }

        let urls = url.split(',').filter(Boolean)
  
        urls.forEach((u, i) => {
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid + i} style={_style_} span={card.width}>
              <div className="video-wrap" style={card.style}>
                <Video card={card} poster={poster} value={u}/>
              </div>
            </div>
          )
        })
      } else if (card.eleType === 'qrcode') {
        let val = ''
  
        if (card.datatype === 'static') {
          val = card.value
        } else if (data.hasOwnProperty(card.field)) {
          val = data[card.field] || ''
        }
  
        if (!val && card.noValue === 'hide') { // 空值隐藏
          return null
        }
  
        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
            <div style={card.style}>
              <div style={{minHeight: card.qrWidth || 50}}>
                {val ? <QrCode card={card} value={val}/> : null}
              </div>
            </div>
          </div>
        )
      } else if (card.eleType === 'currentDate') {
        let val = moment().format(card.dateFormat || 'YYYY-MM-DD')
        
        if (card.fixStyle === 'alone') {
          let _s = {fontSize: card.fixSize, color: card.fixColor, marginLeft: card.fixLeft, marginRight: card.fixRight}
          val = <><span style={_s}>{card.prefix || ''}</span>{val}<span style={_s}>{card.postfix || ''}</span></>
        } else {
          val = `${card.prefix || ''}${val}${card.postfix || ''}`
        }
  
        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
            <div style={card.style}>
              <div className="ant-mk-text line1" style={{height: card.innerHeight || 'auto'}}>{val}</div>
            </div>
          </div>
        )
      } else if (card.eleType === 'formula') {
        let val = 0
        let _style = {...card.style}
        
        if (card.eval === 'func') {
          let _data = []
          if (card.$sync) {
            _data = this.props.syncData
          } else if (data && !data.$$empty) {
            _data = [data]
          }

          let _val = card.formula
          if (/@username@|@fullName@|@bid@/ig.test(_val)) {
            _val = _val.replace(/@username@/ig, sessionStorage.getItem('User_Name') || '').replace(/@fullName@/ig, sessionStorage.getItem('Full_Name') || '').replace(/@bid@/ig, data.$$BID || '')
          }

          try {
            // eslint-disable-next-line
            let func = new Function('data', _val)
            val = func(_data)
          } catch (e) {
            console.warn(e)
            val = ''
          }

          if (!val && card.noValue === 'hide') { // 空值隐藏
            return null
          } else if (typeof(val) === 'object' && val.type === 'linkmenu') {
            // type: 'linkmenu', linkThdMenu: null, menuId: '', value: ``, defaultValue: '', onclick: 'inner'
            let item = {linkType: 'linkmenu', linkThdMenu: val.linkThdMenu}
            let _val_ = val.value || ''

            if (!item.linkThdMenu && val.menuId) {
              item.linkThdMenu = window.GLOB.mkThdMenus.get(val.menuId) || ''
            }
            if (!item.linkThdMenu && val.defaultValue) {
              _val_ = val.defaultValue
            }

            if (val.onclick === 'inner') {
              contents.push(
                <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
                  <div style={_style}>
                    <div className={'ant-mk-text line' + (card.height || '')} style={{height: card.innerHeight}}>
                      <span onClick={(e) => {this.openNewView(e, item)}} dangerouslySetInnerHTML={{__html: _val_}}></span>
                    </div>
                  </div>
                </div>
              )
            } else {
              _style.cursor = 'pointer'
              contents.push(
                <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
                  <div style={_style} onClick={(e) => {this.openNewView(e, item)}}>
                    <div className={'ant-mk-text line' + (card.height || '')} style={{height: card.innerHeight}} dangerouslySetInnerHTML={{__html: _val_}}></div>
                  </div>
                </div>
              )
            }
          } else {
            contents.push(
              <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
                <div style={_style}>
                  <div className={'ant-mk-text line' + (card.height || '')} style={{height: card.innerHeight}} dangerouslySetInnerHTML={{__html: val}}></div>
                </div>
              </div>
            )
          }
          return
        } else if (card.$sync) {
          if (card.eval === 'false') {
            val = ''
          }
          this.props.syncData.forEach(item => {
            let _val = card.formula
            Object.keys(item).forEach(key => {
              let reg = new RegExp('@' + key + '@', 'ig')
              _val = _val.replace(reg, item[key])
            })
            if (card.eval !== 'false') {
              try {
                // eslint-disable-next-line
                _val = eval(_val)
              } catch (e) {
                window.mkInfo(_val)
                console.warn(e)
                _val = 0
              }
            }
  
            // if (!val) return
  
            val += _val
          })
        } else if (data && data.$$empty && /@.*@/.test(card.formula)) {
          val = ''
        } else if (data) {
          let _val = card.formula
          let _data = {}
          Object.keys(data).forEach(key => {
            if (/^\$/.test(key)) return
            if (key === 'children') return
            if (/^1949-10-01/.test(data[key])) {
              _data[key.toLowerCase()] = ''
            } else if (/\d{4}-\d{2}-\d{2}\s00:00:00/.test(data[key])) {
              _data[key.toLowerCase()] = data[key].replace(/\s00:00:00/g, '')
            } else {
              _data[key.toLowerCase()] = data[key]
            }
          })
          _data.username = sessionStorage.getItem('User_Name') || ''
          _data.fullname = sessionStorage.getItem('Full_Name') || ''
          _data.bid = data.$$BID || ''

          if (card.eval === 'false' && card.noValue === 'hide') { // 空值隐藏
            if (card.$keys) {
              let empty = true

              card.$keys.forEach(key => {
                if (!_data.hasOwnProperty(key)) {
                  empty = false
                } else if (_data[key]) {
                  empty = false
                }
              })
  
              if (empty) return null
            }

            Object.keys(_data).forEach(key => {
              if (_data[key]) return

              _val = _val.replace(new RegExp('[^@]*@' + key + '@', 'ig'), '')
            })

            if (!_val) return null
          }

          Object.keys(_data).forEach(key => {
            let reg = new RegExp('@' + key + '@', 'ig')
            _val = _val.replace(reg, _data[key])
          })

          if (card.eval !== 'false') {
            try {
              // eslint-disable-next-line
              _val = eval(_val)
            } catch (e) {
              window.mkInfo(_val)
              console.warn(e)
              _val = ''
            }
          }
  
          val = _val === undefined ? '' : _val
        }

        if (!val && card.noValue === 'hide') { // 空值隐藏
          return null
        }

        if (card.round && typeof(val) === 'number') {
          val = Math.round(val * card.round) / card.round
          val = val.toFixed(card.decimal)
        }
  
        if (val !== '') {
          if (val && typeof(val) === 'string') {
            if (!card.evalchars || card.evalchars.includes('enter')) {
              val = val.replace(/\\n|\n/ig, '<br/>')
            }
            if ((!card.evalchars || card.evalchars.includes('space')) && !/<(span|div|p|a|img)\s/g.test(val)) {
              val = val.replace(/\s/ig, '&nbsp;')
            }

            val = <span dangerouslySetInnerHTML={{__html: val}}></span>
          }
  
          if (card.fixStyle === 'alone') {
            let _s = {fontSize: card.fixSize, color: card.fixColor, marginLeft: card.fixLeft, marginRight: card.fixRight}
            val = <><span style={_s}>{card.prefix || ''}</span>{val}<span style={_s}>{card.postfix || ''}</span></>
          } else {
            val = <>{card.prefix || ''}{val}{card.postfix || ''}</>
          }
        } else if (card.fixStyle === 'alone') {
          let _s = {fontSize: card.fixSize, color: card.fixColor, marginLeft: card.fixLeft, marginRight: card.fixRight}
          val = <span style={_s}>{card.prefix || ''}{card.postfix || ''}</span>
        }

        let className = ''
        if (card.marks) {
          let mark = getMark(card.marks, data, _style)
  
          if (mark.icon) {
            if (mark.position === 'front') {
              val = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {val}</span>
            } else {
              val = <span>{val} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
            }
          } else if (mark.space) {
            val = <><span style={{float: 'left'}} dangerouslySetInnerHTML={{__html: mark.space}}></span>{val}</>
          } else if (mark.point) {
            if (mark.position === 'front') {
              val = <>{mark.point}{val}</>
            } else {
              val = <>{val}{mark.point}</>
            }
          }
          className = mark.signType
        }

        if (card.link && !data.$disabled) {
          _style.cursor = 'pointer'
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div style={_style} onClick={(e) => {this.openNewView(e, card)}}>
                {card.alignItems ? <TextCell card={card} className={'ant-mk-text line' + (card.height || '') + className} value={val}/> : 
                <div className={'ant-mk-text line' + (card.height || '') + className} style={{height: card.innerHeight}}>{val}</div>}
              </div>
            </div>
          )
        } else {
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div style={_style}>
                {card.alignItems ? <TextCell card={card} className={'ant-mk-text line' + (card.height || '') + className} value={val}/> : 
                <div className={'ant-mk-text line' + (card.height || '') + className} style={{height: card.innerHeight}}>{val}</div>}
              </div>
            </div>
          )
        }
      } else if (card.eleType === 'tag') {
        let vals = ''
  
        if (card.datatype === 'static') {
          vals = card.value
        } else {
          vals = data[card.field] || ''
        }
  
        if (!vals && card.noValue === 'hide') { // 空值隐藏
          return null
        }

        vals = vals.split(',').filter(Boolean)
  
        if (card.signs) {
          vals = vals.map(val => {
            let sign = card.signs.filter(s => s.value === val)[0]
            let cell = {value: val, style: {...card.style}}

            if (sign) {
              cell.style.backgroundColor = sign.background
              cell.style.color = sign.color
              cell.style.borderColor = sign.border

              // delete cell.style.borderTopColor
              // delete cell.style.borderBottomColor
              // delete cell.style.borderLeftColor
              // delete cell.style.borderRightColor
            }

            return cell
          })

          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div className="ant-mk-tag">
                {vals.map((item, index) => <span key={index} className="tag-item" style={item.style}>{item.value}</span>)}
              </div>
            </div>
          )
        } else {
          contents.push(
            <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              <div className="ant-mk-tag">
                {vals.map((val, index) => <span key={index} className="tag-item" style={card.style}>{val}</span>)}
              </div>
            </div>
          )
        }
      } else if (card.eleType === 'color') {
        let color = ''
  
        if (card.datatype === 'static') {
          color = card.value
        } else {
          color = data[card.field] || ''
        }
  
        if (!color && card.noValue === 'hide') { // 空值隐藏
          return null
        }

        let _bgstyle = {backgroundColor: color}
  
        if (PicRadio[card.lenWidRadio]) {
          _bgstyle.paddingTop = PicRadio[card.lenWidRadio]
        } else {
          _bgstyle.paddingTop = '100%'
        }

        if (card.copyable === 'true') {
          _bgstyle.cursor = 'pointer'
        }

        contents.push(
          <div className={'ant-col ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
            <div className="ant-mk-color" style={card.style}>
              <div style={_bgstyle} onClick={(e) => {
                if (card.copyable === 'true') {
                  e.stopPropagation()

                  let oInput = document.createElement('input')
                  oInput.value = color
                  document.body.appendChild(oInput)
                  oInput.select()
                  document.execCommand('Copy')
                  document.body.removeChild(oInput)

                  message.success('复制成功。')
                }
              }}></div>
            </div>
          </div>
        )
      } else if (card.eleType === 'button') {
        let _disabled = data.$disabled
        let _data = [data]
  
        if (data.$$type === 'extendCard') {
          _data = data.$$selectedData || []
          if (card.Ot === 'notRequired' && _data.length === 0) {
            _data = [data]
          }
        } else if (card.$sync) {
          _data = this.props.syncData || []
        } else if (data.$$empty) {
          _data = []
        }
        let name = ''
        if (card.showName) {
          name = data[card.showName] || ' '
        }

        _style_ = _style_ || {}
        if (card.wrapStyle) {
          _style_ = {..._style_, ...card.wrapStyle}
        }

        let MkButton = null
        let lid = (data.$$uuid || '') + (data.$Index || '')
  
        if (['exec', 'prompt', 'pop', 'form'].includes(card.OpenType)) {
          MkButton = <NormalButton
            btn={card}
            name={name}
            BID={data.$$BID}
            LID={lid}
            BData={data.$$BData || ''}
            disabled={_disabled}
            setting={cards.setting}
            columns={cards.columns}
            selectedData={_data}
          />
        } else if (card.OpenType === 'excelIn') {
          MkButton = <ExcelInButton
            btn={card}
            BID={data.$$BID}
            LID={lid}
            BData={data.$$BData || ''}
            disabled={_disabled}
            setting={cards.setting}
            selectedData={_data}
          />
        } else if (card.OpenType === 'excelOut') {
          MkButton = <ExcelOutButton
            btn={card}
            BID={data.$$BID}
            LID={lid}
            BData={data.$$BData || ''}
            disabled={_disabled}
            setting={cards.setting}
            selectedData={_data}
          />
        } else if (card.OpenType === 'popview') {
          MkButton = <PopupButton
            btn={card}
            name={name}
            BID={data.$$BID}
            LID={lid}
            BData={data.$$BData || ''}
            disabled={_disabled}
            setting={cards.setting}
            selectedData={_data}
          />
        } else if (card.OpenType === 'tab') {
          MkButton = <TabButton
            btn={card}
            name={name}
            BID={data.$$BID}
            LID={lid}
            BData={data.$$BData || ''}
            disabled={_disabled}
            selectedData={_data}
          />
        } else if (card.OpenType === 'innerpage') {
          MkButton = <NewPageButton
            btn={card}
            name={name}
            BID={data.$$BID}
            LID={lid}
            columns={cards.columns}
            BData={data.$$BData || ''}
            disabled={_disabled}
            selectedData={_data}
          />
        } else if (card.OpenType === 'funcbutton') {
          if (card.funcType === 'changeuser' || card.funcType === 'closetab') {
            MkButton = <ChangeUserButton
              btn={card}
              BID={data.$$BID}
              LID={lid}
              BData={data.$$BData || ''}
              disabled={_disabled}
              setting={cards.setting}
              selectedData={_data}
            />
          } else if (card.funcType === 'print') {
            MkButton = <PrintButton
              btn={card}
              BID={data.$$BID}
              LID={lid}
              BData={data.$$BData || ''}
              disabled={_disabled}
              setting={cards.setting}
              columns={cards.columns}
              selectedData={_data}
            />
          } else if (card.funcType === 'megvii') {
            MkButton = <FuncMegvii
              btn={card}
              BID={data.$$BID}
              LID={lid}
              disabled={_disabled}
              setting={cards.setting}
              selectedData={_data}
            />
          } else if (card.funcType === 'filezip') {
            MkButton = <FuncZip
              btn={card}
              BID={data.$$BID}
              LID={lid}
              BData={data.$$BData || ''}
              disabled={_disabled}
              setting={cards.setting}
              selectedData={_data}
            />
          } else if (card.funcType === 'expPdf') {
            MkButton = <ExportPdf
              btn={card}
              LID={lid}
            />
          } else if (card.funcType === 'addline' || card.funcType === 'delline') {
            MkButton = <EditLine
              btn={card}
              disabled={_disabled}
              selectedData={_data}
            />
          } else {
            MkButton = <FuncButton
              BID={data.$$BID}
              disabled={_disabled}
              LID={lid}
              btn={card}
              columns={cards.columns}
              selectedData={_data}
            />
          }
        }

        if (MkButton) {
          contents.push(
            <div className={'ant-col mk-cell-btn ant-col-' + card.width} key={card.uuid} style={_style_} span={card.width}>
              {MkButton}
            </div>
          )
        }
      }
    })

    return contents
  }

  render() {
    const { cardCell } = this.props
    
    return (
      <div className={'card-cell-list ' + (cardCell && cardCell.setting && cardCell.setting.layout === 'flex' ? 'mk-flex' : '')}>
        {this.getContent()}
        <Col style={{display: 'none'}} span={24}></Col>
      </div>
    )
  }
}

export default CardCellComponent